# pid.pm v1.0 2005/07/13
#
# Copyright 2005 Stephan Otto
#
# Contact software@stephan-otto.net
#
#
# Module PID
# ==========
#
#
# Description
# -----------
#
# Perl module to manage PID files.
#
#
# License
# -------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
# USA.
#
#
# Prerequisites
# -------------
#
# This module uses the modules File::Basename and Proc::ProcessTable.
#
#
# Installation
# ------------
#
# This file needs to be installed somewhere in the Perl @INC module
# search path.
#
#

package pid;

use File::Basename 'basename';
use Proc::ProcessTable;

$pid=0;        # PID
$start=0;      # program start
$tolerance=60; # tolerance in seconds

#
# generate PID file name
#
sub filename { $_[0]?$_[0]:"/var/run/".basename($0).".pid"; }

#
# get PID
#
sub pid { $_[0]?$_[0]:$$; }

#
# write PID to file
#
sub write
{
    my $filename=filename($_[0]);
    my $pid=pid($_[1]);

    open(PID, ">$filename");

    print(PID "$pid\n");

    close(PID);
}

#
# remove PID file
#
sub remove
{
    my $filename=filename($_[0]);

    if(-f "$filename")
    {
	unlink("$filename");
    }
}

#
# read PID and start from file
#
sub read
{
    $pid=0;
    $start=0;

    my $filename=filename($_[0]);
    
    if(-f "$filename")
    {
	open(PID, "$filename");

	$pid=<PID>;

	chomp($pid);

	$start=(stat(PID))[9];

	close(PID);
    }
    
    return $pid;
}

#
# find program with PID and start read from file
#
sub find
{
    if(pid::read($_[0]))
    {
	my $pt=new Proc::ProcessTable;

	foreach my $p (@{$pt->table})
	{
	    if($p->pid==$pid&&abs($p->start-$start)<=$tolerance)
	    {
		return $pid;
	    }
	}

	$pid=0;
	$start=0;
    }

    return $pid;
}

1;

